/*
 * data.c
 *
 * Copyright 2022 Jonathan Schöbel <jonathan@Ubermos-2019>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 *
 *
 */


#include <errno.h>
#include <stdbool.h>
#include <stdlib.h>
#include <string.h>

#include "macro.h"
#include "error.h"
#include "log.h"

#include "validator.h"

#include "data.h"



static inline bool init_validator (struct SH_Data * data,
				   struct SH_Error * error);
static inline void init_pages (struct SH_Data * data);

struct SH_Data *
SH_Data_new (struct SH_Error * error)
{
	struct SH_Data * data;
	data = malloc (sizeof (struct SH_Data));

	if (data == NULL)
	{
		ERROR1 ("Memory allocation for SH_Data failed.\n");

		if (error != NULL)
		{
			error->type = ALLOCATION_FAILED;
		}

		return NULL;
	}

	if (!init_validator (data, error)) return NULL;
	init_pages (data);

	if (error != NULL)
	{
		error->type = SUCCESS;
	}

	return data;
}

static inline void free_validator (struct SH_Data * data);
static inline void free_pages (struct SH_Data * data);

void
SH_Data_free (struct SH_Data * data, struct SH_Error * error)
{
	free_validator (data);
	free_pages (data);

	free (data);

	if (error != NULL)
	{
		error->type = SUCCESS;
	}

	return;
}

static inline bool
init_validator (struct SH_Data * data, struct SH_Error * error)
{
	data->validator = SH_Validator_new (error);

	if (data->validator == NULL)
	{
		return FALSE;
	}

	SH_Validator_register_tag (data->validator, "html", NULL);
	SH_Validator_register_tag (data->validator, "head", NULL);
	SH_Validator_register_tag (data->validator, "body", NULL);
	SH_Validator_register_tag (data->validator, "meta", NULL);
	SH_Validator_register_tag (data->validator, "link", NULL);
	SH_Validator_register_tag (data->validator, "title", NULL);
	SH_Validator_register_tag (data->validator, "main", NULL);
	SH_Validator_register_tag (data->validator, "article", NULL);
	SH_Validator_register_tag (data->validator, "section", NULL);
	SH_Validator_register_tag (data->validator, "h1", NULL);
	SH_Validator_register_tag (data->validator, "h2", NULL);
	SH_Validator_register_tag (data->validator, "h3", NULL);
	SH_Validator_register_tag (data->validator, "h4", NULL);
	SH_Validator_register_tag (data->validator, "h5", NULL);
	SH_Validator_register_tag (data->validator, "h6", NULL);
	SH_Validator_register_tag (data->validator, "p", NULL);
	SH_Validator_register_tag (data->validator, "br", NULL);
	SH_Validator_register_tag (data->validator, "i", NULL);
	SH_Validator_register_tag (data->validator, "b", NULL);
	SH_Validator_register_tag (data->validator, "strong", NULL);
	SH_Validator_register_tag (data->validator, "em", NULL);
	SH_Validator_register_tag (data->validator, "small", NULL);

	return TRUE;
}

static inline void
free_validator (struct SH_Data * data)
{
	SH_Validator_free (data->validator, NULL);
	return;
}

static inline void
init_pages (struct SH_Data * data)
{
	data->page_n = 0;
	data->pages = malloc (0);
	data->last_page = PAGE_ERR;

	return;
}

static inline void
free_pages (struct SH_Data * data)
{
	unsigned int index;

	for (index = 0; index < data->page_n; index++)
	{
		free (data->pages[index].name);
	}

	free (data->pages);
	return;
}

page_t
SH_Data_register_page (struct SH_Data * data, const char * name,
		       struct SH_Error * error)
{
	/* abort on overflow */
	if (data->page_n == UINT_MAX || data->last_page == PAGE_MAX)
	{
		ERROR1 ("Maximum number of pages reached.\n");

		if (error != NULL)
		{
			error->type = DOMAIN_ERROR;
		}

		return PAGE_ERR;
	}

	data->pages = realloc (data->pages,
			(data->page_n + 1) * sizeof (struct SH_Page));

	if (errno == ENOMEM)
	{
		ERROR1 ("Memory allocation for page data failed.\n");

		if (error != NULL)
		{
			error->type = ALLOCATION_FAILED;
		}

		return PAGE_ERR;
	}

	NEXT_PAGE(data->last_page);

	data->pages[data->page_n].id = data->last_page;
	data->pages[data->page_n].name = strdup (name);

	if (errno == ENOMEM)
	{
		ERROR1 ("Memory allocation for page data failed.\n");

		if (error != NULL)
		{
			error->type = ALLOCATION_FAILED;
		}

		return PAGE_ERR;
	}

	data->page_n++;

	if (error != NULL)
	{
		error->type = SUCCESS;
	}

	return data->last_page;
}
