/*
 * validator.c
 *
 * Copyright 2022 Jonathan Schöbel <jonathan@Ubermos-2019>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 *
 *
 */


#include <errno.h>
#include <stdbool.h>
#include <stdlib.h>
#include <string.h>

#include "macro.h"
#include "error.h"
#include "log.h"

#include "validator.h"


static inline void init_tags (struct SH_Validator * validator);

struct SH_Validator *
SH_Validator_new (struct SH_Error * error)
{
	struct SH_Validator * validator;
	validator = malloc (sizeof (struct SH_Validator));

	if (validator == NULL)
	{
		ERROR1 ("Memory allocation for SH_Validator failed.\n");

		if (error != NULL)
		{
			error->type = ALLOCATION_FAILED;
		}

		return NULL;
	}

	init_tags (validator);

	if (error != NULL)
	{
		error->type = SUCCESS;
	}

	return validator;
}

static inline void free_tags (struct SH_Validator * validator);

void
SH_Validator_free (struct SH_Validator * validator,
		   struct SH_Error * error)
{
	free_tags (validator);
	free (validator);

	if (error != NULL)
	{
		error->type = SUCCESS;
	}

	return;
}

static inline void
init_tags (struct SH_Validator * validator)
{
	validator->tag_n = 0;
	validator->tags = malloc (0);
	validator->last_tag = TAG_ERR;

	return;
}

static inline void
free_tags (struct SH_Validator * validator)
{
	unsigned int index;

	for (index = 0; index < validator->tag_n; index++)
	{
		free (validator->tags[index].name);
	}

	free (validator->tags);
	return;
}

tag_t
SH_Validator_register_tag (struct SH_Validator * validator,
			   const char * tag, struct SH_Error * error)
{
	/* abort on overflow */
	if (validator->tag_n == UINT_MAX
	|| validator->last_tag == TAG_MAX)
	{
		ERROR1 ("Maximum number of tags reached.\n");

		if (error != NULL)
		{
			error->type = DOMAIN_ERROR;
		}

		return TAG_ERR;
	}

	validator->tags = realloc (validator->tags,
			(validator->tag_n + 1) * sizeof (struct SH_Tag));

	if (errno == ENOMEM)
	{
		ERROR1 ("Memory allocation for tag failed.\n");

		if (error != NULL)
		{
			error->type = ALLOCATION_FAILED;
		}

		return TAG_ERR;
	}

	NEXT_TAG(validator->last_tag);

	validator->tags[validator->tag_n].id = validator->last_tag;
	validator->tags[validator->tag_n].name = strdup (tag);

	if (errno == ENOMEM)
	{
		ERROR1 ("Memory allocation for tag failed.\n");

		if (error != NULL)
		{
			error->type = ALLOCATION_FAILED;
		}

		return TAG_ERR;
	}

	validator->tag_n++;

	if (error != NULL)
	{
		error->type = SUCCESS;
	}

	return validator->last_tag;
}

bool
SH_Validator_check_tag (struct SH_Validator * validator,
			const char * tag)
{
	unsigned int index;

	for (index = 0; index < validator->tag_n; index++)
	{
		if (strcmp (validator->tags[index].name, tag) == 0)
		{
			return TRUE;
		}
	}

	return FALSE;
}
